local cmd, g, fn, execute = vim.cmd, vim.g, vim.fn, vim.api_nvim_command
local scopes = {o = vim.o, b = vim.bo, w = vim.wo}

local install_path = vim.fn.stdpath('data') .. '/site/pack/packer/start/packer.nvim'
if vim.fn.empty(vim.fn.glob(install_path)) > 0 then
  execute('!git clone https://github.com/wbthomason/packer.nvim '.. install_path)
end

-- packer
require('packer').startup(function()
  use "wbthomason/packer.nvim"

  use {'lewis6991/gitsigns.nvim', requires = 'nvim-lua/plenary.nvim'}
  use {'nvim-treesitter/nvim-treesitter', run = ':TSUpdate'}
  use {
    'nvim-telescope/telescope.nvim',
    requires = {'nvim-lua/popup.nvim', 'nvim-lua/plenary.nvim'}
  }
  use 'hoob3rt/lualine.nvim'

  use 'neovim/nvim-lspconfig'
  use 'nvim-lua/lsp_extensions.nvim'
  use 'hrsh7th/nvim-compe'

  use {'vlime/vlime', rtp = 'vim/', ft = 'lisp'}

  use 'mhartington/formatter.nvim'
  use {'itspriddle/vim-shellcheck', ft = 'sh'}
  use 'mfussenegger/nvim-dap'
  use {
    'b3nj5m1n/kommentary',
    ft = {
      'rust', 'python', 'haskell', 'c', 'cpp',
      'racket', 'julia', 'javascript', 'r', 'sh',
      'html', 'css'
    }
  }
  use {
    'theHamsta/nvim-dap-virtual-text',
    ft = {'rust', 'python', 'haskell', 'c', 'cpp'}
  }
end)

-- gitsigns
require('gitsigns').setup {
  preview_config = { border = "none" },
  keymaps = {
    noremap = true,

    ['n ]h'] = { expr = true, "&diff ? ']h' : '<cmd>lua require\"gitsigns\".next_hunk()<CR>'"},
    ['n [h'] = { expr = true, "&diff ? '[h' : '<cmd>lua require\"gitsigns\".prev_hunk()<CR>'"},

    ['n <localleader>hs'] = '<cmd>lua require"gitsigns".stage_hunk()<CR>',
    ['v <localleader>hs'] = '<cmd>lua require"gitsigns".stage_hunk({vim.fn.line("."), vim.fn.line("v")})<CR>',
    ['n <localleader>hS'] = '<cmd>lua require"gitsigns".stage_buffer()<CR>',
    ['n <localleader>hu'] = '<cmd>lua require"gitsigns".undo_stage_hunk()<CR>',
    ['n <localleader>hr'] = '<cmd>lua require"gitsigns".reset_hunk()<CR>',
    ['v <localleader>hr'] = '<cmd>lua require"gitsigns".reset_hunk({vim.fn.line("."), vim.fn.line("v")})<CR>',
    ['n <localleader>hR'] = '<cmd>lua require"gitsigns".reset_buffer()<CR>',
    ['n <localleader>hp'] = '<cmd>lua require"gitsigns".preview_hunk()<CR>',
    ['n <localleader>hb'] = '<cmd>lua require"gitsigns".blame_line(true)<CR>',

    ['o ih'] = ':<C-U>lua require"gitsigns".select_hunk()<CR>',
    ['x ih'] = ':<C-U>lua require"gitsigns".select_hunk()<CR>'
  }
}

--telescope
require('telescope').setup {
  defaults = {
    vimgrep_arguments = {
      'rg',
      '--color=never',
      '--no-heading',
      '--with-filename',
      '--line-number',
      '--column',
      '--smart-case'
    },
    selection_caret = "» ",
    entry_prefix = "  ",
    initial_mode = "insert",
    sorting_strategy = "ascending",
    layout_strategy = "vertical",
    file_ignore_patterns = {},
    color_devicons = false,
    use_less = true,
    set_env = {['COLORTERM'] = 'truecolor'},
  },
  pickers = {
    buffers = {
      sort_lastused = true,
      -- theme = "dropdown",
      theme = "ivy",
      layout_config = {height = 10},
      previewer = false,
      mappings = {
        i = {["<c-d>"] = require("telescope.actions").delete_buffer},
        n = {["<c-d>"] = require("telescope.actions").delete_buffer}
      }
    },
    find_files = {
      theme = "ivy",
      layout_config = {height = 10},
      previewer = false
    },
    oldfiles = {
      sort_lastused = true,
      theme = "ivy",
      layout_config = {height = 10},
      previewer = false
    },
    command_history = {
      sort_lastused = true,
      theme = "ivy",
      layout_config = {height = 10},
      previewer = false
    },
    current_buffer_fuzzy_find = {
      previewer = false
    }
  }
}

-- nvim-compe
require'compe'.setup {
  autocomplete = false;
  min_length = 3;
  documentation = true;
  source = {
    path = true;
    buffer = true;
    nvim_lsp = true
  }
}

-- treesitter
require'nvim-treesitter.configs'.setup {
  ensure_installed = "maintained",
  highlight = {
    enable = true,
  },
  rainbow = {
    enable = true,
    extended_mode = true
  }
}

-- dap
local dap = require('dap')

dap.adapters.python = {
  type = 'executable';
  command = '/usr/bin/python3';
  args = { '-m', 'debugpy.adapter' };
}
dap.configurations.python = {
  {
    type = 'python';
    request = 'launch';
    name = "Launch file";
    -- https://github.com/microsoft/debugpy/wiki/Debug-configuration-settings
    program = "${file}";
    pythonPath = function()
      local cwd = vim.fn.getcwd()
      if vim.fn.executable(cwd .. '/venv/bin/python') == 1 then
        return cwd .. '/venv/bin/python'
      elseif vim.fn.executable(cwd .. '/.venv/bin/python') == 1 then
        return cwd .. '/.venv/bin/python'
      elseif vim.fn.executable(cwd .. '/.env/bin/python') == 1 then
        return cwd .. '/.env/bin/python'
      else
        return '/usr/bin/python3'
      end
    end;
  }
}

dap.adapters.haskell = {
  type = 'executable';
  command = 'haskell-debug-adapter';
  args = {'--hackage-version=0.0.33.0'};
}
dap.configurations.haskell = {
  {
    type = 'haskell',
    request = 'launch',
    name = 'Debug',
    workspace = '${workspaceFolder}',
    startup = "${file}",
    stopOnEntry = true,
    logFile = vim.fn.stdpath('data') .. '/haskell-dap.log',
    logLevel = 'WARNING',
    ghciEnv = vim.empty_dict(),
    ghciPrompt = "λ: ",
    ghciInitialPrompt = "λ: ",
    ghciCmd= "stack ghci --test --no-load --no-build --main-is TARGET --ghci-options -fprint-evld-with-show",
  }
}

dap.adapters.lldb = {
  type = 'executable',
  command = '/usr/bin/lldb-vscode-10',
  name = "lldb"
}
dap.configurations.cpp = {
  {
    name = "Launch",
    type = "lldb",
    request = "launch",
    program = function()
      return vim.fn.input('Path to executable: ', vim.fn.getcwd() .. '/', 'file')
    end,
    cwd = '${workspaceFolder}',
    stopOnEntry = false,
    args = {},
    runInTerminal = false,
  }
}
dap.configurations.c = dap.configurations.cpp
dap.configurations.rust = dap.configurations.cpp

vim.g.dap_virtual_text = true

-- lualine
require('lualine').setup {
  options = {
    theme = 'nord',
    section_separators = {'', ''},
    component_separators = {'', ''},
    disabled_filetypes = {},
    icons_enabled = false,
  },
  sections = {
    lualine_a = {},
    lualine_b = {'filename', {'branch', color = {bg = '#4C566A', fg = '#81A1C1'}}},
    lualine_c = {{'diagnostics', sources = {'nvim_lsp'}, sections = {'error', 'warn', 'info'}}},
    lualine_x = {'encoding', 'filetype'},
    lualine_y = {'progress', color = {bg = '#eee8d5'}},
    lualine_z = {}
  },
  extensions = {'quickfix'}
}

-- additional settings
require('lsp')
require('format')
require('mappings')
require('settings')

-- colorscheme
vim.cmd[[set background=light]]
vim.cmd[[colorscheme plain]]
